#!/s/std/bin/env perl

# This script automates the process of generating a set of evolved
# sequences using specific evolution parameters for seq-gen and
# sgEvolver, aligning those sequences, and scoring the alignments


use strict;
use POSIX;
if( @ARGV != 2 ){
	die "Usage: simujobrun.pl <job description file> <mauve|mavid|mlagan>";
}

# Set the location of the evolution and scoring tools here:
# this is the directory where seq-gen, sgEvolver, and scoreAlignment reside
my $tools_dir="/u/d/a/darling/mauve/";
my $mauve_dir="/u/d/a/darling/mauve/";
my $mavid_dir="/u/d/a/darling/mavid/";
my $lagan_dir="/u/d/a/darling/lagan121/";
my $xmfa_alignment = "";

# an array of files that need to be deleted when cleaning up
my @delete_files = ();

# ROADMAP:
# 1) simulate evolution according to parameters
# 2) align genomes using the selected aligner
# 3) score alignments

#
# read in the job specification
#
my $job_spec_file = $ARGV[0];
open( JOB_FILE, "<$job_spec_file" );

my $job_line;
chomp( my $sequence_length = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $nt_sub_scale = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $gamma_shape = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $indel_rate = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $small_ht_rate = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $small_ht_size = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $large_ht_rate = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $large_ht_min = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $large_ht_max = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $inv_rate = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $inv_size = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $nt_a_freq = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $nt_c_freq = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $nt_g_freq = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $nt_t_freq = (split( /=/, $job_line = <JOB_FILE> ))[1] );

chomp( my $tree_filename = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $ancestral_donor = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $ancestral_seq_name = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $donor_seq_name = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $seqgen_out_name = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $evolved_seqs_name = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $evolved_seqs_fname = (split( /=/, $job_line = <JOB_FILE> ))[1] );

chomp( my $ancestral_start = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $donor_start = (split( /=/, $job_line = <JOB_FILE> ))[1] );

chomp( my $seqgen_random = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $seqgen_donor_random = (split( /=/, $job_line = <JOB_FILE> ))[1] );
chomp( my $sgEvolver_random = (split( /=/, $job_line = <JOB_FILE> ))[1] );

		# write command lines executed a file
open( COMLINES, ">command_lines.txt"  );


# generate an ancestral base
open( ANCESTRAL, ">$ancestral_seq_name" );
print ANCESTRAL " 1 $sequence_length\n";
print ANCESTRAL "Ancestral\n";
close ANCESTRAL;
	
my $rval;

		# extract the specified ancestral sequence
my $extract_cl = "dd if=$ancestral_donor bs=1 skip=$ancestral_start count=$sequence_length";
executeCommand( $extract_cl, ">$ancestral_seq_name", "ancestral_dd.err" );

my $sgEvolver_random = int(rand(0x7FFFFFFF));

		# generate a data set
my $seqgen_cl = $tools_dir."seq-gen -mHKY -a $gamma_shape -i 0 -z $seqgen_random -f ";
$seqgen_cl .= "$nt_a_freq $nt_c_freq $nt_g_freq $nt_t_freq ";
$seqgen_cl .= "-t .5 -k 1 -wa -on -s $nt_sub_scale $tree_filename ";
$seqgen_cl .= "< $ancestral_seq_name ";
executeCommand( $seqgen_cl, ">$seqgen_out_name", "seqgen_ancestral.err" );

		# generate a donor base
open( DONOR, ">$donor_seq_name" );
print DONOR " 1 $sequence_length\n";
print DONOR "Donor\n";
close DONOR;


		# extract the specified donor sequence
$extract_cl = "dd if=$ancestral_donor bs=1 skip=$donor_start count=$sequence_length";
executeCommand( $extract_cl, ">$donor_seq_name", "donor_dd.err" );

		# generate a donor data set
$seqgen_cl = $tools_dir."seq-gen -mHKY -a $gamma_shape -i 0 -z $seqgen_donor_random  -f ";
$seqgen_cl .= "$nt_a_freq $nt_c_freq $nt_g_freq $nt_t_freq ";
$seqgen_cl .= "-t .5 -k 1 -wa -on -s $nt_sub_scale $tree_filename ";
$seqgen_cl .= "< $donor_seq_name ";
executeCommand( $seqgen_cl, ">$seqgen_out_name", "seqgen_donor.err" );

		# continue evolution with sgEvolver
my $sgevolver_cl = $tools_dir."sgEvolver --indel-freq=".$indel_rate." --small-ht-freq=$small_ht_rate ".
"--large-ht-freq=$large_ht_rate --inversion-freq=$inv_rate --large-ht-min=$large_ht_min --large-ht-max=$large_ht_max".
"--random-seed=$sgEvolver_random".
" --inversion-size=$inv_size $tree_filename $seqgen_out_name $evolved_seqs_name $evolved_seqs_fname";
$rval = executeCommand( $sgevolver_cl, "sgEvolver.out", "sgEvolver.err" );

# die if sgEvolver failed
die "Failure in sgEvolver" if( $rval != 0 );

# delete seq-gen related files
push( @delete_files, $seqgen_out_name );
push( @delete_files, $ancestral_seq_name );
push( @delete_files, $donor_seq_name );

deleteFiles( @delete_files );

#
# STEP 2:  align genomes using the selected aligner
#

alignMauve() if( $ARGV[1] eq "mauve" );
alignMavid() if( $ARGV[1] eq "mavid" );
alignMlagan() if( $ARGV[1] eq "mlagan" );
alignSlagan() if( $ARGV[1] eq "slagan" );

#
# STEP 3: score the alignment
#

my $scoreAlignment_path = $tools_dir."scoreAlignment";

my $score_cl = "$scoreAlignment_path $evolved_seqs_name $xmfa_alignment $evolved_seqs_fname";
executeCommand( $score_cl, "scores.txt", "scoring.err" );

# do a backbone scoring
my $bb_file = "backbone.dat";
push( @delete_files, $bb_file );
my $extractbb_cl = $tools_dir."extractBackbone $evolved_seqs_fname $evolved_seqs_name 50 50 $bb_file";
executeCommand( $extractbb_cl, "extract_bb.out", "extract_bb.err" );

$score_cl = "$scoreAlignment_path $bb_file $xmfa_alignment $evolved_seqs_fname";
executeCommand( $score_cl, "bb_scores.txt", "bb_scoring.err" );

#
# delete alignment files
#
#system( "rm $evolved_seqs_name" );
#system( "rm $evolved_seqs_fname" );

push( @delete_files, $evolved_seqs_name );
push( @delete_files, $evolved_seqs_fname );

deleteFiles( @delete_files );
exit(0);

sub executeCommand {
  my $command = shift;
  my $stdout_file = shift;
  my $stderr_file = shift;
  $command .= " >$stdout_file 2>$stderr_file";
  print "Executing $command\n";
  print COMLINES "$command\n";
  my $rval = system($command);
  `echo "Exited with code $rval" >> $stderr_file` if $rval != 0;
  return $rval;
}

sub deleteFiles {
  foreach(@_){
    `rm -f $_`;
    print COMLINES "rm -f $_\n";
  }
}

sub alignMauve {
  $xmfa_alignment = "aligned.dat";
  push( @delete_files, $xmfa_alignment );
  push( @delete_files, "aligned.mums" );
  push( @delete_files, "aligned.mauve" );
  my $mauve_cl = $mauve_dir."mauveAligner --mums -o aligned.mums $evolved_seqs_fname";
  executeCommand( $mauve_cl, "mums.out", "mums.err" );

  $mauve_cl = $mauve_dir."mauveAligner -i aligned.mums -o aligned.mauve ".
  "--output-alignment=$xmfa_alignment $evolved_seqs_fname";
  $rval = executeCommand( $mauve_cl, "aligner.out", "aligner.err" );

  # die if mauve failed
  die "Failure in aligner" if( $rval != 0 );
}

sub alignMavid {
  $xmfa_alignment = "mavid.dat";

  # dust the sequence for mavid
  my $dust_cl = "$mavid_dir/mdust $evolved_seqs_fname";
  executeCommand( $dust_cl, "$evolved_seqs_fname.masked", "mdust.err" );

  my $mavid_cl = $mavid_dir."/mavid $tree_filename $evolved_seqs_fname";
  executeCommand( $mavid_cl, "mavid.out", "mavid.err" );


  # step 2.1 convert mavid.mfa to xmfa format
  my $xmfa_cl = "$tools_dir/mfa2xmfa mavid.mfa $xmfa_alignment";
  executeCommand( $xmfa_cl, "mfa2xmfa.out", "mfa2xmfa.err" );

  push( @delete_files, "$evolved_seqs_fname.masked" );
  push( @delete_files, "mavid.mfa" );
  push( @delete_files, "mavid.phy" );
  push( @delete_files, "$xmfa_alignment" );

}


sub alignMlagan {

}

sub alignSlagan {

}

